# __init__.py
from .carga_datos import cargar_csv_desde_url, guardar_csv_local
from .conteo_periodo import contar_por_anio_trimestre_o_mes, graficar_conteo_barras, graficar_conteo_lineas
from .tasa import tasa_crecimiento, graficar_tasa
from .perfil_victimas import perfil_de_victimas, graficar_perfil_barras, graficar_perfil_lineas



# tasa_crecimiento.py

import pandas as pd
import matplotlib.pyplot as plt


def tasa_crecimiento(df, periodo="anio", desde=None, hasta=None):
    
     # Validación de entrada
    if periodo not in ["anio", "trimestre", "mes"]:
        raise ValueError("El parámetro 'periodo' debe ser 'anio', 'trimestre' o 'mes'")
    if desde is None or hasta is None:
        raise ValueError("Debes especificar los parámetros 'desde' y 'hasta'")

     # Agrupación según el periodo
    if periodo == "anio":
        grupo = df.groupby("anio").size()
        valor_desde = grupo.get(desde[0], 0)
        valor_hasta = grupo.get(hasta[0], 0)

    elif periodo == "trimestre":
        grupo = df.groupby(["anio", "trimestre"]).size()
        valor_desde = grupo.get((desde[0], desde[1]), 0)
        valor_hasta = grupo.get((hasta[0], hasta[1]), 0)

    elif periodo == "mes":
        grupo = df.groupby(["anio", "mes"]).size()
        valor_desde = grupo.get((desde[0], desde[1]), 0)
        valor_hasta = grupo.get((hasta[0], hasta[1]), 0)
    # Calcular tasa de crecimiento:
    if valor_desde == 0:
        raise ZeroDivisionError(f"No hay datos en el periodo desde {desde} (división por cero)")

    tasa = ((valor_hasta - valor_desde) / valor_desde) * 100
    return tasa

def graficar_tasa (valor_desde, valor_hasta, desde, hasta, tasa, periodo="anio"):
    etiquetas = [f"{desde}", f"{hasta}"]
    valores = [valor_desde, valor_hasta]

    plt.figure(figsize=(8, 5))
    plt.plot(etiquetas, valores, marker='o', color='blue', linewidth=2)
    plt.title(f"Tasa de crecimiento entre {desde} y {hasta}\nCrecimiento: {tasa:.2f}%")
    plt.xlabel(f"Periodo ({periodo})")
    plt.ylabel("Número de incidentes")
    plt.grid(True)
    plt.tight_layout()
    plt.show()


#perfil_victimas.py

import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt


def perfil_de_victimas(df, periodo="anio", victima="peaton"):
    # Validación de entrada
    if periodo not in ["anio", "trimestre", "mes"]:
        raise ValueError("El parámetro 'periodo' debe ser 'anio', 'trimestre' o 'mes'")
    if victima not in ["peaton", "ciclista", "pasajero", "motociclista", "conductor"]:
        raise ValueError("El parámetro 'victima' debe ser 'peaton', 'ciclista', 'pasajero', 'motociclista' o 'conductor'")
        
    lesionados_col = f"{victima}_lesionado"
    fallecidos_col = f"{victima}_fallecido"

    #validar columnas
    if periodo not in df.columns:
        raise ValueError(f"La columna '{periodo}' no existe en el DataFrame")
    if lesionados_col not in df.columns or fallecidos_col not in df.columns:
        raise ValueError(f"Las columnas '{lesionados_col}' y/o '{fallecidos_col}' no existen en el DataFrame")


    # Agrupamos por el periodo y sumamos lesionados y fallecidos
    grupo = df.groupby(periodo).agg(
        lesionados=(lesionados_col, 'sum'),
        fallecidos=(fallecidos_col, 'sum')
    )

    # Convertimos a diccionario con formato {periodo: {'lesionados': X, 'fallecidos': Y}}
    perfil = grupo.to_dict(orient='index')

    # Convertir valores a enteros
    for key, value in perfil.items():
        perfil[key]['lesionados'] = int(value['lesionados'])
        perfil[key]['fallecidos'] = int(value['fallecidos'])

    return perfil

def graficar_perfil_barras (perfil_dict, periodo="anio", victima="peaton"):
    # Convertir e dic a  DF
    df = pd.DataFrame.from_dict(perfil_dict, orient='index').reset_index()
    df.rename(columns={'index': periodo}, inplace=True)

    # Convertir a string si el periodo es mes o trimestre (por legibilidad)
    if periodo in ["mes", "trimestre"]:
        df[periodo] = df[periodo].astype(str)

    # Reorganizar a formato largo para seaborn
    df_largo = df.melt(id_vars=[periodo], value_vars=['lesionados', 'fallecidos'], 
                       var_name='tipo', value_name='cantidad')

    ### gráfica de barras
    plt.figure(figsize=(12, 6))
    sns.barplot(data=df_largo, x=periodo, y='cantidad', hue='tipo', palette='Set2')

    plt.title(f"Lesionados y fallecidos ({victima}) por {periodo}")
    plt.xlabel(periodo.capitalize())
    plt.ylabel("Número de víctimas")
    plt.xticks(rotation=45)
    plt.tight_layout()
    plt.legend(title="Tipo de víctima")
    plt.show()

def graficar_perfil_lineas(perfil_dict, periodo="anio", victima="peaton"):
    # Convertir el diccionario en DataFrame
    df = pd.DataFrame.from_dict(perfil_dict, orient='index').reset_index()
    df.rename(columns={'index': periodo}, inplace=True)

    # Convertir a string si el periodo es mes o trimestre (por legibilidad)
    if periodo in ["mes", "trimestre"]:
        df[periodo] = df[periodo].astype(str)

    # Reorganizar a formato largo para seaborn
    df_largo = df.melt(
        id_vars=[periodo],
        value_vars=['lesionados', 'fallecidos'],
        var_name='tipo',
        value_name='cantidad'
    )

    # Graficar líneas
    plt.figure(figsize=(12, 6))
    sns.lineplot(data=df_largo, x=periodo, y='cantidad', hue='tipo', marker="o", palette='Set1'
    )

    plt.title(f"Lesionados y fallecidos ({victima}) por {periodo}")
    plt.xlabel(periodo.capitalize())
    plt.ylabel("Número de víctimas")
    plt.xticks(rotation=45)
    plt.tight_layout()
    plt.legend(title="Tipo de víctima")
    plt.show()



# conteo_periodo.py

import pandas as pd
import seaborn as sns
import matplotlib.pyplot as plt

def contar_por_anio_trimestre_o_mes(df, periodo="trimestre"):
       
    # Verificar si las columnas necesarias están en el DataFrame
    if 'anio' not in df.columns or ('trimestre' not in df.columns and 'mes' not in df.columns):
        raise ValueError("El DataFrame debe contener las columnas 'anio', 'trimestre' y/o 'mes'")
    
    # Agrupar por 'anio', 'anio y trimestre' o 'anio y mes'
    if periodo == "anio":
        conteo = df.groupby(['anio']).size().reset_index(name="conteo")
    elif periodo == "trimestre":
        conteo = df.groupby(['anio', 'trimestre']).size().reset_index(name="conteo")
    elif periodo == "mes":
        conteo = df.groupby(['anio', 'mes']).size().reset_index(name="conteo")
    else:
        raise ValueError("El parámetro 'periodo' debe ser 'anio', 'trimestre' o 'mes'")

    return conteo

def graficar_conteo_barras (conteo_df, periodo="trimestre"):
    plt.figure(figsize=(12, 6))
    
    if periodo == "anio":
        sns.barplot(data=conteo_df, x="anio", y="conteo", palette="Blues_d")
        plt.title("Conteo de incidentes por año")
    
    elif periodo == "trimestre":
        conteo_df["periodo"] = conteo_df["anio"].astype(str) + " T" + conteo_df["trimestre"].astype(str)
        sns.barplot(data=conteo_df, x="periodo", y="conteo", palette="Greens_d")
        plt.title("Conteo de incidentes por trimestre")
        plt.xticks(rotation=45)
    
    elif periodo == "mes":
        conteo_df["periodo"] = conteo_df["anio"].astype(str) + "-" + conteo_df["mes"].astype(str).str.zfill(2)
        sns.barplot(data=conteo_df, x="periodo", y="conteo", palette="Oranges_d")
        plt.title("Conteo de incidentes por mes")
        plt.xticks(rotation=45)

    plt.xlabel("Periodo")
    plt.ylabel("Número de incidentes")
    plt.tight_layout() #ajusta los elemtos de la grafica para que no se empalmen
    plt.show()


def graficar_conteo_lineas (conteo_df, periodo="trimestre"):
    plt.figure(figsize=(12, 6))
    
    if periodo == "anio":
        sns.lineplot(data=conteo_df, x="anio", y="conteo", marker="o", color="blue")
        plt.title("Conteo de incidentes por año")
    
    elif periodo == "trimestre":
        conteo_df["periodo"] = conteo_df["anio"].astype(str) + " T" + conteo_df["trimestre"].astype(str)
        sns.lineplot(data=conteo_df, x="periodo", y="conteo", marker="o", color="green")
        plt.title("Conteo de incidentes por trimestre")
        plt.xticks(rotation=45)
    
    elif periodo == "mes":
        conteo_df["periodo"] = conteo_df["anio"].astype(str) + "-" + conteo_df["mes"].astype(str).str.zfill(2)
        sns.lineplot(data=conteo_df, x="periodo", y="conteo", marker="o", color="orange")
        plt.title("Conteo de incidentes por mes")
        plt.xticks(rotation=45)

    plt.xlabel("Periodo")
    plt.ylabel("Número de incidentes")
    plt.tight_layout() #ajusta los elemtos de la grafica para que no se empalmen
    plt.grid(True)
    plt.show()
